@@ -13,7 +13,6 @@ module m_make_result
   public:: fstr_reorder_node_beam
   public:: setup_contact_output_variables
 
-
 contains
 
   !C***
@@ -80,7 +79,7 @@ contains
     call hecmw_result_init( hecMESH, istep, header, comment )
 
     ! --- TIME
-    id = 3 !global data
+    id = HECMW_RESULT_DTYPE_GLOBAL
     label = 'TOTALTIME'
     work(1) = time
     call hecmw_result_add( id, 1, label, work )
@@ -88,7 +87,7 @@ contains
     ! --- DISPLACEMENT
     if( fstrSOLID%output_ctrl(3)%outinfo%on(1) ) then
       if(ndof == 4) then
-        id = 1
+        id = HECMW_RESULT_DTYPE_NODE
         ! for VELOCITY
         nitem = n_comp_valtype( fstrSOLID%output_ctrl(3)%outinfo%vtype(1), 3 )
         allocate( unode(3*hecMESH%n_node) )
@@ -112,7 +111,7 @@ contains
         call hecmw_result_add( id, nitem, label, unode )
         deallocate( unode )
       else if(ndof == 6) then
-        id = 1
+        id = HECMW_RESULT_DTYPE_NODE
         nitem = n_comp_valtype( fstrSOLID%output_ctrl(3)%outinfo%vtype(1), 3 )
         allocate( unode(hecMESH%n_node*3) )
         unode = 0.0d0
@@ -129,7 +128,7 @@ contains
         call hecmw_result_add( id, nitem, label, unode )
         deallocate( unode )
       else
-        id = 1
+        id = HECMW_RESULT_DTYPE_NODE
         nitem = n_comp_valtype( fstrSOLID%output_ctrl(3)%outinfo%vtype(1), ndof )
         allocate( unode(hecMESH%n_node*ndof) )
         unode = 0.0d0
@@ -153,7 +152,7 @@ contains
     ! --- ROTATION
     if (fstrSOLID%output_ctrl(3)%outinfo%on(18)) then
       if(ndof == 6) then
-        id = 1
+        id = HECMW_RESULT_DTYPE_NODE
         nitem = n_comp_valtype( fstrSOLID%output_ctrl(3)%outinfo%vtype(1), 3 )
         label = 'ROTATION'
         allocate( rnode(hecMESH%n_node*3) )
@@ -171,7 +170,7 @@ contains
         deallocate( rnode )
       else
         if ( is_33shell == 1) then
-          id = 1
+          id = HECMW_RESULT_DTYPE_NODE
           nitem = n_comp_valtype( fstrSOLID%output_ctrl(3)%outinfo%vtype(1), ndof )
           label = 'ROTATION'
           allocate( rnode(hecMESH%n_node*ndof) )
@@ -185,7 +184,7 @@ contains
 
     ! --- VELOCITY
     if( is_dynamic .and. fstrSOLID%output_ctrl(3)%outinfo%on(15) ) then
-      id = 1
+      id = HECMW_RESULT_DTYPE_NODE
       nitem = n_comp_valtype( fstrSOLID%output_ctrl(3)%outinfo%vtype(15), ndof )
       label = 'VELOCITY'
       call hecmw_result_add( id, nitem, label, fstrDYNAMIC%VEL(:,idx) )
@@ -193,7 +192,7 @@ contains
 
     ! --- ACCELERATION
     if( is_dynamic .and. fstrSOLID%output_ctrl(3)%outinfo%on(16) ) then
-      id = 1
+      id = HECMW_RESULT_DTYPE_NODE
       nitem = n_comp_valtype( fstrSOLID%output_ctrl(3)%outinfo%vtype(16), ndof )
       label = 'ACCELERATION'
       call hecmw_result_add( id, nitem, label, fstrDYNAMIC%ACC(:,idx) )
@@ -201,7 +200,7 @@ contains
 
     ! --- REACTION FORCE
     if( fstrSOLID%output_ctrl(3)%outinfo%on(2) ) then
-      id = 1
+      id = HECMW_RESULT_DTYPE_NODE
       nitem = n_comp_valtype( fstrSOLID%output_ctrl(3)%outinfo%vtype(2), ndof )
       label = 'REACTION_FORCE'
       call hecmw_result_add( id, nitem, label, fstrSOLID%REACTION )
@@ -233,7 +232,7 @@ contains
     !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
     ! --- STRAIN @gauss
     if( fstrSOLID%output_ctrl(3)%outinfo%on(9) .and. ndof/=6 ) then
-      id = 2
+      id = HECMW_RESULT_DTYPE_ELEM
       nitem = n_comp_valtype( fstrSOLID%output_ctrl(3)%outinfo%vtype(9), ndof )
       ngauss = fstrSOLID%maxn_gauss
       work(:) = 0.d0
@@ -256,7 +255,7 @@ contains
 
     ! --- STRESS @gauss
     if( fstrSOLID%output_ctrl(3)%outinfo%on(10) .and. ndof/=6 ) then
-      id = 2
+      id = HECMW_RESULT_DTYPE_ELEM
       nitem = n_comp_valtype( fstrSOLID%output_ctrl(3)%outinfo%vtype(10), ndof )
       ngauss = fstrSOLID%maxn_gauss
       work(:) = 0.d0
@@ -279,7 +278,7 @@ contains
 
     ! --- PLASTIC STRAIN @gauss
     if( fstrSOLID%output_ctrl(3)%outinfo%on(11) .and. fstrSOLID%StaticType/=3 ) then
-      id = 2
+      id = HECMW_RESULT_DTYPE_ELEM
       nitem = n_comp_valtype( fstrSOLID%output_ctrl(3)%outinfo%vtype(11), ndof )
       ngauss = fstrSOLID%maxn_gauss
       do k = 1, ngauss
@@ -299,7 +298,7 @@ contains
 
     ! --- THERMAL STRAIN @node
     if( fstrSOLID%output_ctrl(3)%outinfo%on(12) .and. associated(tnstrain) ) then
-      id = 1
+      id = HECMW_RESULT_DTYPE_NODE
       nitem = n_comp_valtype( fstrSOLID%output_ctrl(3)%outinfo%vtype(12), ndof )
       label = 'THERMAL_NodalSTRAIN'
       call hecmw_result_add( id, nitem, label, tnstrain )
@@ -307,7 +306,7 @@ contains
 
     ! --- THERMAL STRAIN @element
     if( fstrSOLID%output_ctrl(3)%outinfo%on(13) .and. associated(testrain) ) then
-      id = 2
+      id = HECMW_RESULT_DTYPE_ELEM
       nitem = n_comp_valtype( fstrSOLID%output_ctrl(3)%outinfo%vtype(13), ndof )
       label = 'THERMAL_ElementalSTRAIN'
       call hecmw_result_add( id, nitem, label, testrain )
@@ -315,7 +314,7 @@ contains
 
     ! --- THERMAL STRAIN @gauss
     if( fstrSOLID%output_ctrl(3)%outinfo%on(14) .and. associated(testrain) ) then
-      id = 2
+      id = HECMW_RESULT_DTYPE_ELEM
       nitem = n_comp_valtype( fstrSOLID%output_ctrl(3)%outinfo%vtype(14), ndof )
       ngauss = fstrSOLID%maxn_gauss
       do k = 1, ngauss
@@ -339,7 +338,7 @@ contains
 
     ! --- YIELD RATIO
     if( fstrSOLID%output_ctrl(3)%outinfo%on(29) ) then
-      id = 2
+      id = HECMW_RESULT_DTYPE_ELEM
       nitem = n_comp_valtype( fstrSOLID%output_ctrl(3)%outinfo%vtype(29), ndof )
       label = "YIELD_RATIO"
       call hecmw_result_add( id, nitem, label, yield_ratio )
@@ -347,7 +346,7 @@ contains
 
     ! --- CONTACT NORMAL FORCE @node
     if( fstrSOLID%output_ctrl(3)%outinfo%on(30) .and. associated(fstrSOLID%CONT_NFORCE) ) then
-      id = 1
+      id = HECMW_RESULT_DTYPE_NODE
       nitem = n_comp_valtype( fstrSOLID%output_ctrl(3)%outinfo%vtype(30), ndof )
       label = 'CONTACT_NFORCE'
       call hecmw_result_add( id, nitem, label, fstrSOLID%CONT_NFORCE )
@@ -355,7 +354,7 @@ contains
 
     ! --- CONTACT FRICTION FORCE @node
     if( fstrSOLID%output_ctrl(3)%outinfo%on(31) .and. associated(fstrSOLID%CONT_FRIC) ) then
-      id = 1
+      id = HECMW_RESULT_DTYPE_NODE
       nitem = n_comp_valtype( fstrSOLID%output_ctrl(3)%outinfo%vtype(31), ndof )
       label = 'CONTACT_FRICTION'
       call hecmw_result_add( id, nitem, label, fstrSOLID%CONT_FRIC )
@@ -363,7 +362,7 @@ contains
 
     ! --- CONTACT RELATIVE VELOCITY @node
     if( fstrSOLID%output_ctrl(3)%outinfo%on(32) .and. associated(fstrSOLID%CONT_RELVEL) ) then
-      id = 1
+      id = HECMW_RESULT_DTYPE_NODE
       nitem = n_comp_valtype( fstrSOLID%output_ctrl(3)%outinfo%vtype(32), ndof )
       label = 'CONTACT_RELVEL'
       call hecmw_result_add( id, nitem, label, fstrSOLID%CONT_RELVEL )
@@ -371,7 +370,7 @@ contains
 
     ! --- CONTACT STATE @node
     if( fstrSOLID%output_ctrl(3)%outinfo%on(33) .and. associated(fstrSOLID%CONT_STATE) ) then
-      id = 1
+      id = HECMW_RESULT_DTYPE_NODE
       nitem = n_comp_valtype( fstrSOLID%output_ctrl(3)%outinfo%vtype(33), ndof )
       label = 'CONTACT_STATE'
       call hecmw_result_add( id, nitem, label, fstrSOLID%CONT_STATE )
@@ -379,7 +378,7 @@ contains
 
     ! --- CONTACT NORMAL TRACTION @node
     if( fstrSOLID%output_ctrl(3)%outinfo%on(36) .and. associated(fstrSOLID%CONT_NTRAC) ) then
-      id = 1
+      id = HECMW_RESULT_DTYPE_NODE
       nitem = n_comp_valtype( fstrSOLID%output_ctrl(3)%outinfo%vtype(36), ndof )
       label = 'CONTACT_NTRACTION'
       call hecmw_result_add( id, nitem, label, fstrSOLID%CONT_NTRAC )
@@ -387,7 +386,7 @@ contains
 
     ! --- CONTACT FRICTION TRACTION @node
     if( fstrSOLID%output_ctrl(3)%outinfo%on(37) .and. associated(fstrSOLID%CONT_FTRAC) ) then
-      id = 1
+      id = HECMW_RESULT_DTYPE_NODE
       nitem = n_comp_valtype( fstrSOLID%output_ctrl(3)%outinfo%vtype(37), ndof )
       label = 'CONTACT_FTRACTION'
       call hecmw_result_add( id, nitem, label, fstrSOLID%CONT_FTRAC )
@@ -443,12 +442,14 @@ contains
     character(len=6)                :: clyr
     character(len=4)                :: cnum
     integer(kind=kint) :: i, j, k, ndof, mdof, id, nitem, nn, mm, ngauss, it
+    real(kind=kreal), allocatable   :: work(:)
 
     ndof = hecMESH%n_dof
+    allocate( work(hecMESH%n_elem) )
 
     ! --- STRAIN @node
     if (fstrSOLID%output_ctrl(3)%outinfo%on(3)) then
-      id = 1
+      id = HECMW_RESULT_DTYPE_NODE
       nitem = n_comp_valtype( fstrSOLID%output_ctrl(3)%outinfo%vtype(3), ndof )
       label = 'NodalSTRAIN'//trim(clyr)
       call hecmw_result_add( id, nitem, label, RES%STRAIN )
@@ -456,7 +457,7 @@ contains
 
     ! --- STRESS @node
     if( fstrSOLID%output_ctrl(3)%outinfo%on(4) ) then
-      id = 1
+      id = HECMW_RESULT_DTYPE_NODE
       nitem = n_comp_valtype( fstrSOLID%output_ctrl(3)%outinfo%vtype(4), ndof )
       label = 'NodalSTRESS'//trim(clyr)
       call hecmw_result_add( id, nitem, label, RES%STRESS )
@@ -464,7 +465,7 @@ contains
 
     ! --- MISES @node
     if( fstrSOLID%output_ctrl(3)%outinfo%on(5) ) then
-      id = 1
+      id = HECMW_RESULT_DTYPE_NODE
       nitem = n_comp_valtype( fstrSOLID%output_ctrl(3)%outinfo%vtype(5), ndof )
       label = 'NodalMISES'//trim(clyr)
       call hecmw_result_add( id, nitem, label, RES%MISES )
@@ -472,7 +473,7 @@ contains
 
     ! --- NODAL PRINC STRESS
     if( fstrSOLID%output_ctrl(3)%outinfo%on(19) ) then
-      id = 1
+      id = HECMW_RESULT_DTYPE_NODE
       nitem = n_comp_valtype( fstrSOLID%output_ctrl(3)%outinfo%vtype(19), ndof )
       label = 'NodalPrincipalSTRESS'//trim(clyr)
       call hecmw_result_add( id, nitem, label, RES%PSTRESS )
@@ -480,7 +481,7 @@ contains
 
     ! --- NODAL PRINC STRAIN
     if( fstrSOLID%output_ctrl(3)%outinfo%on(21) ) then
-      id = 1
+      id = HECMW_RESULT_DTYPE_NODE
       nitem = n_comp_valtype( fstrSOLID%output_ctrl(3)%outinfo%vtype(21), ndof )
       label = 'NodalPrincipalSTRAIN'//trim(clyr)
       call hecmw_result_add( id, nitem, label, RES%PSTRAIN )
@@ -488,7 +489,7 @@ contains
 
     ! --- NODAL PRINC STRESS VECTOR
     if( fstrSOLID%output_ctrl(3)%outinfo%on(23) ) then
-      id = 1
+      id = HECMW_RESULT_DTYPE_NODE
       do k=1,3
         write(cnum,'(i0)')k
         nitem = n_comp_valtype( fstrSOLID%output_ctrl(3)%outinfo%vtype(23), ndof )
@@ -499,7 +500,7 @@ contains
 
     ! --- NODAL PRINC STRAIN VECTOR
     if( fstrSOLID%output_ctrl(3)%outinfo%on(25) ) then
-      id = 1
+      id = HECMW_RESULT_DTYPE_NODE
       do k=1,3
         write(cnum,'(i0)')k
         nitem = n_comp_valtype( fstrSOLID%output_ctrl(3)%outinfo%vtype(25), ndof )
@@ -511,7 +512,7 @@ contains
     !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
     ! --- STRAIN @element
     if( fstrSOLID%output_ctrl(3)%outinfo%on(6) ) then
-      id = 2
+      id = HECMW_RESULT_DTYPE_ELEM
       nitem = n_comp_valtype( fstrSOLID%output_ctrl(3)%outinfo%vtype(6), ndof )
       label = 'ElementalSTRAIN'//trim(clyr)
       call hecmw_result_add( id, nitem, label, RES%ESTRAIN )
@@ -519,7 +520,7 @@ contains
 
     ! --- STRESS @element
     if( fstrSOLID%output_ctrl(3)%outinfo%on(7) ) then
-      id = 2
+      id = HECMW_RESULT_DTYPE_ELEM
       nitem = n_comp_valtype( fstrSOLID%output_ctrl(3)%outinfo%vtype(7), ndof )
       label = 'ElementalSTRESS'//trim(clyr)
       call hecmw_result_add( id, nitem, label, RES%ESTRESS )
@@ -527,7 +528,7 @@ contains
 
     ! --- NQM @element
     if( fstrSOLID%output_ctrl(3)%outinfo%on(35) ) then
-      id = 2
+      id = HECMW_RESULT_DTYPE_ELEM
       nitem = n_comp_valtype( fstrSOLID%output_ctrl(3)%outinfo%vtype(35), ndof )
       label = 'ElementalNQM'//trim(clyr)
       call hecmw_result_add( id, nitem, label, RES%ENQM )
@@ -535,7 +536,7 @@ contains
 
     ! --- MISES @element
     if( fstrSOLID%output_ctrl(3)%outinfo%on(8)) then
-      id = 2
+      id = HECMW_RESULT_DTYPE_ELEM
       nitem = n_comp_valtype( fstrSOLID%output_ctrl(3)%outinfo%vtype(8), ndof )
       label = 'ElementalMISES'//trim(clyr)
       call hecmw_result_add( id, nitem, label, RES%EMISES )
@@ -543,7 +544,7 @@ contains
 
     ! --- Principal_STRESS @element
     if( fstrSOLID%output_ctrl(3)%outinfo%on(20) ) then
-      id = 2
+      id = HECMW_RESULT_DTYPE_ELEM
       nitem = n_comp_valtype( fstrSOLID%output_ctrl(3)%outinfo%vtype(20), ndof )
       label = 'ElementalPrincipalSTRESS'//trim(clyr)
       call hecmw_result_add( id, nitem, label, RES%EPSTRESS )
@@ -551,7 +552,7 @@ contains
 
     ! --- Principal_STRAIN @element
     if( fstrSOLID%output_ctrl(3)%outinfo%on(22) ) then
-      id = 2
+      id = HECMW_RESULT_DTYPE_ELEM
       nitem = n_comp_valtype( fstrSOLID%output_ctrl(3)%outinfo%vtype(22), ndof )
       label = 'ElementalPrincipalSTRAIN'//trim(clyr)
       call hecmw_result_add( id, nitem, label, RES%EPSTRAIN )
@@ -559,7 +560,7 @@ contains
 
     ! --- ELEM PRINC STRESS VECTOR
     if( fstrSOLID%output_ctrl(3)%outinfo%on(24) ) then
-      id = 2
+      id = HECMW_RESULT_DTYPE_ELEM
       do k=1,3
         write(cnum,'(i0)')k
         nitem = n_comp_valtype( fstrSOLID%output_ctrl(3)%outinfo%vtype(24), ndof )
@@ -570,7 +571,7 @@ contains
 
     !ELEM PRINC STRAIN VECTOR
     if( fstrSOLID%output_ctrl(3)%outinfo%on(26) ) then
-      id = 2
+      id = HECMW_RESULT_DTYPE_ELEM
       do k=1,3
         write(cnum,'(i0)')k
         nitem = n_comp_valtype( fstrSOLID%output_ctrl(3)%outinfo%vtype(26), ndof )
@@ -579,6 +580,24 @@ contains
       end do
     endif
 
+    ! --- PLASTIC STRAIN @element
+    if( fstrSOLID%output_ctrl(3)%outinfo%on(43) ) then
+      id = 2
+      nitem = n_comp_valtype( fstrSOLID%output_ctrl(3)%outinfo%vtype(43), ndof )
+      ngauss = fstrSOLID%maxn_gauss
+      label = 'ElementalPLSTRAIN'//trim(clyr)
+      do i = 1, hecMESH%n_elem
+        work(i) = 0.d0
+        do j = 1, size(fstrSOLID%elements(i)%gausses) 
+          work(i) = work(i) + fstrSOLID%elements(i)%gausses(j)%plstrain
+        enddo
+        work(i) = work(i) / size(fstrSOLID%elements(i)%gausses) 
+        RES%EPLSTRAIN(i) = work(i)
+      enddo
+      call hecmw_result_add( id, nitem, label, work )
+    endif
+    deallocate( work )
+  
   end subroutine fstr_write_result_main
 
   !C***
@@ -803,6 +822,11 @@ contains
       ecomp = ecomp + 3
       eitem = eitem + 3*n_comp_valtype( fstrSOLID%output_ctrl(4)%outinfo%vtype(26), ndof )
     endif
+    ! --- PLASTIC STRAIN @element
+    if( fstrSOLID%output_ctrl(4)%outinfo%on(43) ) then
+      ecomp = ecomp + 1
+      eitem = eitem + n_comp_valtype( fstrSOLID%output_ctrl(4)%outinfo%vtype(43), ndof )
+    endif
     ! --- MATERIAL @element
     if( fstrSOLID%output_ctrl(4)%outinfo%on(34) ) then
       ecomp = ecomp + 1
@@ -1420,6 +1444,23 @@ contains
       enddo
     endif
 
+    ! --- PLSTRAIN @elem
+    if(fstrSOLID%output_ctrl(4)%outinfo%on(43)) then
+      ecomp = ecomp + 1
+      nn = n_comp_valtype( fstrSOLID%output_ctrl(4)%outinfo%vtype(43), ndof )
+      fstrRESULT%ne_dof(ecomp) = nn
+      fstrRESULT%elem_label(ecomp) = 'ElementalPLSTRAIN'
+      do i = 1, hecMESH%n_elem
+        RES%EPLSTRAIN(i) = 0.d0
+        do j = 1, size(fstrSOLID%elements(i)%gausses) 
+          RES%EPLSTRAIN(i) = RES%EPLSTRAIN(i) + fstrSOLID%elements(i)%gausses(j)%plstrain
+        enddo
+        RES%EPLSTRAIN(i) = RES%EPLSTRAIN(i) / size(fstrSOLID%elements(i)%gausses)
+        fstrRESULT%elem_val_item(eitem*(i-1)+1+jitem) = RES%EPLSTRAIN(i)
+      enddo
+      jitem = jitem + nn
+    endif
+
   end subroutine fstr_make_result_main
 
   subroutine fstr_reorder_node_shell(fstrSOLID, hecMESH, unode)
@@ -1639,69 +1680,4 @@ contains
 
   end subroutine
 
-  subroutine fstr_setup_parancon_contactvalue(hecMESH,ndof,vec,vtype)
-  use m_fstr
-  implicit none
-  type(hecmwST_local_mesh), intent(in)      :: hecMESH
-  integer(kind=kint), intent(in)            :: ndof
-  real(kind=kreal), pointer, intent(inout)  :: vec(:)
-  integer(kind=kint), intent(in)            :: vtype !1:value, 2:state
-  !
-  real(kind=kreal) ::  rhsB
-  integer(kind=kint) ::  i,j,N,i0,N_loc,nndof
-  integer(kind=kint) :: offset, pid, lid
-  integer(kind=kint), allocatable :: displs(:)
-  real(kind=kreal), allocatable   :: vec_all(:)
-
-  !
-  N_loc = hecMESH%nn_internal
-  allocate(displs(0:nprocs))
-  displs(:) = 0
-  displs(myrank+1) = N_loc
-  call hecmw_allreduce_I(hecMESH, displs, nprocs+1, hecmw_sum)
-  do i=1,nprocs
-    displs(i) = displs(i-1) + displs(i)
-  end do
-  offset = displs(myrank)
-  N = displs(nprocs)
-
-  allocate(vec_all(ndof*N))
-
-  if( vtype == 1 ) then
-    vec_all(:) = 0.d0
-    do i= hecMESH%nn_internal+1,hecMESH%n_node
-      pid = hecMESH%node_ID(i*2)
-      lid = hecMESH%node_ID(i*2-1)
-      i0 = (displs(pid) + (lid-1))*ndof
-      vec_all(i0+1:i0+ndof) = vec((i-1)*ndof+1:i*ndof)
-      vec((i-1)*ndof+1:i*ndof) = 0.d0
-    enddo
-
-    call hecmw_allreduce_R(hecMESH, vec_all, N*ndof, hecmw_sum)
-
-    do i=1,ndof*N_loc
-      vec(i) = vec(i) + vec_all(offset*ndof+i)
-    end do
-  else if( vtype == 2 ) then
-    vec_all(:) = -1000.d0
-    do i= hecMESH%nn_internal+1,hecMESH%n_node
-      if( vec(i) == 0.d0 ) cycle
-      pid = hecMESH%node_ID(i*2)
-      lid = hecMESH%node_ID(i*2-1)
-      i0 = displs(pid) + lid
-      vec_all(i0) = vec(i)
-    enddo
-
-    call hecmw_allreduce_R(hecMESH, vec_all, N, hecmw_max)
-
-    do i=1,N_loc
-      if( vec_all(offset+i) == -1000.d0 ) cycle
-      if( vec(i) < vec_all(offset+i) ) vec(i) = vec_all(offset+i)
-    end do
-  end if
-
-  deallocate(displs,vec_all)
-  end subroutine
-
-
 end module m_make_result
